class ObjectGraphTraverser

  # Params:
  # +object_graph+:: a hash of class name => association pairs, e.g.
  #   { 'Foo' => ['bars'], 'Bar' => ['baz'] }
  def initialize(object_graph)
    @graph = object_graph
  end

  # Given a starting object, traverses the graph calling the given block on each node.
  #
  # Params:
  # +object+:: root object from which to start traversing
  # +parent+:: optional parent object
  # +block+:: a block that will be called on each node, receiving the node, its parent and the graph of objects under the current node.
  def traverse_graph(object,parent=nil,&block)
    nodes = fetch_object_graph(object.class).map do |node|
      Array(object.send(node.to_sym)).each do |node_object|
        traverse_graph(node_object,object,&block)
      end
      node
    end
    block.call(object,parent,nodes)
  end

  private

  def fetch_object_graph(klass)
    nodes = @graph.fetch(klass.name) {[]}
    if nodes.empty? && klass.superclass
      nodes = fetch_object_graph(klass.superclass)
    end
    nodes
  end

end
